/* 

                          Firewall Builder

                 Copyright (C) 2000 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: IPv4Dialog.cc,v 1.4 2002/09/04 02:26:54 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"
#include "IPv4Dialog.hh"
#include "IPAddressWidget.hh"
#include "helpers.hh"
#include "MessageDialog.hh"

#include "fwbuilder/dns.hh"

using namespace libfwbuilder;


IPv4Dialog::IPv4Dialog(FWObject *obj)
{
    object=obj; 

    object_parameters.push_back( DialogOption( ipv4_name  , "name"     ) );
    object_parameters.push_back( DialogOption( comment    , "comment"  ) );

    obj_addr=manage(new IPAddressWidget());
    addr_table->attach(*obj_addr, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
    obj_addr->changed.connect(SigC::slot(this,&IPv4Dialog::on_changed));

    obj_netm=manage(new IPAddressWidget());
    addr_table->attach(*obj_netm, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
    obj_netm->changed.connect(SigC::slot(this,&IPv4Dialog::on_changed));

    obj_addr->setAssociatedNetmaskWidget( obj_netm );

    object_parameters.push_back( DialogOption( obj_addr  , "address"  ) );
    object_parameters.push_back( DialogOption( obj_netm  , "netmask"  ) );

}

IPv4Dialog::~IPv4Dialog()
{
}


void IPv4Dialog::on_changed()
{  
    data_changed_flag(true);
}

void IPv4Dialog::on_dns_clicked()
{
    vector<IPAddress> addr;

    try {
        addr=DNS::getHostByName( ipv4_name->get_text() );
    } catch (FWException &e)
    {
        MessageDialog::Error( e.toString(), this );
        return;
    }

    obj_addr->setAddress( addr.front() );
}

void IPv4Dialog::wrk2dlg()
{
    loadPageOptions(object,  object_parameters );
    ipv4_name->grab_focus();
}

bool IPv4Dialog::dlg2wrk()
{

/*
 *  check if name is valid
 */
    if ( ! checkObjectName(ipv4_name->get_text()) )      return(false);
    if ( ! checkIPaddress( obj_addr->getAsString() ) )  return(false);
    if ( ! checkNetmask( obj_netm->getAsString() ) )    return(false);

    savePageOptions(object,  object_parameters );
    return(true);
}

