/* 

                          Firewall Builder

                 Copyright (C) 2000 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NewFirewallDruid.cc,v 1.6 2002/09/25 20:36:53 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "fwbuilder/Firewall.hh"
#include "fwbuilder/Interface.hh"
#include "fwbuilder/IPv4.hh"

#include "main_window.hh"

#include "NewFirewallDruid.hh"
#include "MessageDialog.hh"

#include "NewFirewallDruidP10Dialog.hh"
#include "NewFirewallDruidP20Dialog.hh"

#include "FWObjectBook.hh"
#include "FWObjectDatabaseGUI.hh"

#include <gtk--.h>

#include "glademm_support.hh"

#include <iostream>


using namespace libfwbuilder;


enum {
    PAGE_10,
    PAGE_20,
    PAGE_F,
} NewFirewallDruidPages;


NewFirewallDruid::NewFirewallDruid(main_window *mw,
                           const string &t,
                           const string& l,
                           Firewall  *_fw) : Druid(t,l)
{
    main_w=mw;
    fw=_fw;

    set_default_size(500,450);
    set_position(GTK_WIN_POS_CENTER);

/************************************************************************
 *    Page #10
 */
    p10=manage(new NewFirewallDruidP10Dialog());
    createStandardPage(PAGE_10,p10);

/************************************************************************
 *    Page #20
 */
    p20=manage(new NewFirewallDruidP20Dialog());
    createStandardPage(PAGE_20,p20);


/************************************************************************
 *    Final Page 
 */
    createFinishPage(PAGE_F,
 _("Click 'Finish' to create and file new object, click 'Cancel' to discard."));

    glademm_set_Widget("NewFirewallDruid", this);

    setPage(PAGE_10);
}


void NewFirewallDruid::on_prepare()
{
    int current_page=get_current_page_id();

    switch (current_page) {
    case -1:
    case PAGE_10:	p10->prepare();	break;
    case PAGE_20:	p20->prepare();	break;
    }
}

gboolean NewFirewallDruid::on_next()
{
    int current_page=get_current_page_id();

    switch (current_page) {

    case -1:
    case PAGE_10:
        if (p10->obj_name->get_text()=="") return true;
        setPage(PAGE_20);
	return true;

    case PAGE_20:
	setPage(PAGE_F);
	return true;
    }

    return true;
}

gboolean NewFirewallDruid::on_back()
{
    int current_page=get_current_page_id();

    switch (current_page) {

    case PAGE_20:
	setPage(PAGE_10);
	return true;

    case PAGE_F:
        setPage(PAGE_20);
	return true;

    }
    return  true;
}

gboolean NewFirewallDruid::on_cancel()
{
    destroy();
    return true;
}


void NewFirewallDruid::on_finish()
{
    fw->setName( p10->obj_name->get_text() );

    for (int i=0; i<p20->iface_list->getSize(); ++i) 
    {
        InterfaceData idata=p20->iface_list->getInterface(i);

        Interface *iface= Interface::cast(FWObjectDatabaseGUI::newInterface(fw->getId()));
        iface->setName( idata.name );
        iface->setStr("physAddress", idata.physicalAddress );
        iface->setDyn(idata.isDyn);
        main_window::insertInAllObjectBooks(iface);

        if (idata.isDyn)
        {
            idata.address="0.0.0.0";
            idata.netmask="0.0.0.0";
        }
        IPv4 *addr=IPv4::cast( FWObjectDatabaseGUI::newIPv4(iface->getId()) );
        addr->setAddress( idata.address );
        addr->setNetmask( idata.netmask );
        main_window::insertInAllObjectBooks(addr);
    }

    try
    {
        main_window::changeLabelInAllObjectBooks(fw->getId());
        FWObjectBook *w=main_w->getObjectBook();
        if (w!=NULL)  w->showObject( fw->getId() );
        main_window::conditionalRefreshAllDialogs(fw);
    } catch (FWException &ex)
    {
        cerr << ex.toString();
    }


    destroy();
}

