/* 

                          Firewall Builder

                 Copyright (C) 2002 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: OSConfigurator_linux24.cc,v 1.22 2002/12/05 06:14:05 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

#include "OSConfigurator_linux24.hh"

#include "fwbuilder/Firewall.hh"
#include "fwbuilder/FWOptions.hh"
#include "fwbuilder/Interface.hh"
#include "fwbuilder/IPv4.hh"
#include "fwbuilder/Network.hh"

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

#include <algorithm>
#include <functional>
#include <iostream>

#include <assert.h>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string OSConfigurator_linux24::myPlatformName() { return "Linux24"; }

OSConfigurator_linux24::OSConfigurator_linux24(FWObjectDatabase *_db,
                                               const string &fwname) : 
    OSConfigurator(_db,fwname) , os_data()
{
}

void OSConfigurator_linux24::processFirewallOptions() 
{
    FWOptions* options=fw->getOptionsObject();
    string s;
    int    i;

/*
 * check if all interfaces configured for the firewall are present
 */
    if (options->getBool("verify_interfaces")) 
    {
        list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
        if ( ! l2.empty() )
        {
            output << endl;
            output << "INTERFACES=\"";
            for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) {
                Interface *iface=Interface::cast(*i);

                output << iface->getName() << " ";
            }
            output << "\"" << endl;
            output << "for i in $INTERFACES ; do" << endl;
            output << "  $IP link show \"$i\" > /dev/null 2>&1 || {" << endl;
            output << "    echo Interface $i does not exist" << endl;
            output << "    exit 1" << endl;
            output << "  }" << endl;
            output << "done" << endl;
            output << endl;
        }
    }

/*
 *    Turn off packet forwarding for now. We'll turn it on if needed in the end
 */
    output << "\n\n";
    output << "FWD=`cat /proc/sys/net/ipv4/ip_forward`\n";
    output << "echo \"0\" > /proc/sys/net/ipv4/ip_forward\n\n";


    s=options->getStr("linux24_ip_dynaddr");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/ip_dynaddr\n\n";


    s=options->getStr("linux24_rp_filter");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/conf/all/rp_filter\n\n";

    s=options->getStr("linux24_accept_source_route");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/conf/all/accept_source_route\n\n";

    s=options->getStr("linux24_accept_redirects");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/conf/all/accept_redirects\n\n";

    s=options->getStr("linux24_log_martians");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/conf/all/log_martians\n\n";


    
    s=options->getStr("linux24_icmp_echo_ignore_broadcasts");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts\n\n";

    s=options->getStr("linux24_icmp_echo_ignore_all");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/icmp_echo_ignore_all\n\n";

    s=options->getStr("linux24_icmp_ignore_bogus_error_responses");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses\n\n";


    
    if ( (i=options->getInt("linux24_tcp_fin_timeout"))>0 )
	output << "echo " << i << " > /proc/sys/net/ipv4/tcp_fin_timeout\n\n";

    if ( (i=options->getInt("linux24_tcp_keepalive_interval"))>0 )
	output << "echo " << i << " > /proc/sys/net/ipv4/tcp_keepalive_intvl\n\n";

    s=options->getStr("linux24_tcp_window_scaling");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_window_scaling\n\n";

    s=options->getStr("linux24_tcp_sack");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_sack\n\n";

    s=options->getStr("linux24_tcp_fack");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_fack\n\n";

    s=options->getStr("linux24_tcp_syncookies");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_syncookies\n\n";

    s=options->getStr("linux24_tcp_ecn");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_ecn\n\n";

    s=options->getStr("linux24_tcp_timestamps");
    if (!s.empty())
	output << "echo " << s << " > /proc/sys/net/ipv4/tcp_timestamps\n\n";




/*
 *  Remove all host static routes and "pub" ARP entries if we are going to 
 *  create new ones
 */
    if ( options->getBool("manage_virtual_addr") ) {
	list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) {
	    Interface *interface_=Interface::cast(*i);
	    if ( interface_->isDyn() ) continue;
	    if ( interface_->isLoopback() ) continue;

            output << "$IP -4 neigh flush dev " 
                   << interface_->getName() << endl;

            output << "$IP -4 addr flush dev " 
                   << interface_->getName() 
                   << " label \"" << interface_->getName() << ":FWB*\""
                   << endl;
	}
	output << endl;
    }

    output << endl;


}

void OSConfigurator_linux24::addVirtualAddressForNAT(const Network &nw)
{
    if (virtual_addresses.empty() || 
	find(virtual_addresses.begin(),virtual_addresses.end(),nw.getAddress())==virtual_addresses.end()) 
    {
        FWObjectTypedChildIterator i=fw->findByType(Interface::TYPENAME);
        for ( ; i!=i.end(); ++i ) 
        {
	    Interface *iface=dynamic_cast<Interface*>(*i);
	    assert(iface);

            if (iface->isDyn()) continue;

	    FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
	    for ( ; j!=j.end(); ++j ) 
            {
		IPv4 *iaddr=IPv4::cast(*j);

                IPNetwork n( nw.getAddress() , nw.getNetmask() );
                if (n.belongs( iaddr->getAddress() ))
                {
                    IPAddress a;
                    string str, subnet, first, last;

                    a=nw.getAddress() +1;
                    first  = a.toString();
                        
                    a      = n.getBroadcastAddress() -1;
                    last   = a.toString();

                    output << "a=\"" << first << "\"" << endl;
                    output << "while test \"$a\" != \"" 
                           << last 
                           << "\"; do" 
                           << endl;

                    output << "  "
                           << "$IP -4 addr add ${a}/" 
                           << nw.getNetmask().getLength()
                           << " dev "
                           << iface->getName() 
                           << " brd + scope global" 
                           << " label " << iface->getName() << ":FWB${va_num}" 
                           << endl;

                    output << "
  a4=${a%%.*}
  s=${a#*.}
  a3=${s%%.*}
  s=${s#*.}
  a2=${s%%.*}
  s=${s#*.}
  a1=$s

  aa=$((16777216 * a4))
  aa=$((aa + 65536 * a3))
  aa=$((aa + 256 * a2))
  aa=$((aa + a1))

  ab=$((aa + 1))

  a1=$((ab % 256))
  ab=$((ab / 256))
  a2=$((ab % 256))
  ab=$((ab / 256))
  a3=$((ab % 256))
  ab=$((ab / 256))
  a4=$((ab % 256))

  a=$a4\".\"$a3\".\"$a2\".\"$a1 " << endl;

                    output << "  va_num=$((va_num + 1))" << endl;
                    output << "done" << endl << endl;
                    virtual_addresses.push_back(nw.getAddress());
                    return;
                }
            }
        }
	throw FWException(_("Can not add virtual address ") + 
                          nw.getAddress().toString() );
    }
}

void OSConfigurator_linux24::addVirtualAddressForNAT(const IPAddress &addr)
{
    if (virtual_addresses.empty() || 
	find(virtual_addresses.begin(),virtual_addresses.end(),addr)==virtual_addresses.end()) 
    {
        FWObjectTypedChildIterator i=fw->findByType(Interface::TYPENAME);
        for ( ; i!=i.end(); ++i ) 
        {
	    Interface *iface=dynamic_cast<Interface*>(*i);
	    assert(iface);

            if (iface->isDyn()) continue;

	    FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
	    for ( ; j!=j.end(); ++j ) 
            {
		IPv4 *iaddr=IPv4::cast(*j);

                IPNetwork n( iaddr->getAddress() , iaddr->getNetmask() );
                if ( n.belongs( addr ) ) 
                {
                    IPNetwork netw(iaddr->getAddress(),iaddr->getNetmask());

                    output << "$IP -4 addr add "
                           << addr.toString()
                           << "/" << iaddr->getNetmask().getLength()
                           << " dev "
                           << iface->getName() << " brd + scope global" 
                           << " label " << iface->getName() << ":FWB${va_num}" 
                           << endl;
                    output << "  va_num=$((va_num + 1))" << endl << endl;
                    virtual_addresses.push_back(addr);
                    return;
                }
            }
        }
	throw FWException(_("Can not add virtual address ") + addr.toString() );
    }
}

void  OSConfigurator_linux24::printPathForAllTools(const string &os)
{
    FWOptions* options=fw->getOptionsObject();
    
    string s, path_modprobe, path_iptables, path_ip, path_logger;

    s=options->getStr("linux24_path_modprobe");
    if (!s.empty()) path_modprobe=s;
    else            path_modprobe=os_data.getPathForTool(os,OSData::MODPROBE);

    s=options->getStr("linux24_path_iptables");
    if (!s.empty()) path_iptables=s;
    else            path_iptables=os_data.getPathForTool(os,OSData::IPTABLES);

    s=options->getStr("linux24_path_ip");
    if (!s.empty()) path_ip=s;
    else            path_ip=os_data.getPathForTool(os,OSData::IP);

    s=options->getStr("linux24_path_logger");
    if (!s.empty()) path_logger=s;
    else            path_logger=os_data.getPathForTool(os,OSData::LOGGER);


    output << "check() {" << endl;
    output << "  if test ! -x \"$1\"; then" << endl;
    output << "    echo \"$1 " << _("not found or is not executable") << "\"" << endl;
    output << "    exit 1" << endl;
    output << "  fi" << endl;
    output << "}" << endl;
    output << endl;
    output << "log() {" << endl;
    output << "  if test -x \"$LOGGER\"; then" << endl;
    output << "    logger -p info \"$1\"" << endl;
    output << "  fi" << endl;
    output << "}" << endl;
    output << endl;
    output << "MODPROBE=\""+path_modprobe+"\"\n";
    output << "IPTABLES=\""+path_iptables+"\"\n";
    output << "IP=\""      +path_ip+"\"\n";
    output << "LOGGER=\""  +path_logger+"\"\n";
    output << endl;

    output << "check $MODPROBE" << endl;
    output << "check $IPTABLES" << endl;
    output << "check $IP" << endl;

    output << endl;
}

int  OSConfigurator_linux24::prolog()
{
    FWOptions* options=fw->getOptionsObject();
    string firewall_dir=options->getStr("firewall_dir");

    char          *timestr;
    time_t         tm;
    struct tm     *stm;

    tm=time(NULL);
    stm=localtime(&tm);
    timestr=strdup(ctime(&tm));
    timestr[ strlen(timestr)-1 ]='\0';
    
    char* user_name=(char*)g_get_user_name();
    if (user_name==NULL) {
        cerr << _("Can't figure out your user name, aborting") << endl;
        exit(1);
    }

    output << _("#\n\
#  This is automatically generated file. DO NOT MODIFY !\n\
#\n\
#  Firewall Builder  fwb_ipt v") << VERSION << "-" <<  RELEASE_NUM << _(" \n\
#\n\
#  Generated ") << timestr << " " << tzname[stm->tm_isdst] << _(" by ") 
            << user_name << "\n#\n#\n#\n#\n";


/*
 * print definitions for variables IPTABLES, IP, LOGGER. Some day we may
 * add a choice of distro in the GUI. Right now paths are either default
 * for a given distro, or custom strings entered by user in the GUI and stored
 * in firewall options.
 */
    printPathForAllTools(DISTRO);


    if ( firewall_dir.empty() ) firewall_dir="/etc";

//    string env_path=options->getStr("script_env_path");
//    if ( ! env_path.empty() )
//        output << "PATH=" << env_path << endl;


    output << "cd " << firewall_dir << " || exit 1" << endl << endl;

    output << "log \"";
    output << _("Activating firewall script generated ")
           << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
           << user_name;
    output << "\"" << endl;

    output << endl;

    output << "va_num=1" << endl;

/*
 * Process firewall options, build OS network configuration script
 */
    processFirewallOptions();


    output << "$IPTABLES -P OUTPUT  DROP" << endl;
    output << "$IPTABLES -P INPUT   DROP" << endl;
    output << "$IPTABLES -P FORWARD DROP" << endl;
    output << endl;

/*
 *    flush all the rules which existed before
 */

    output << "\n\
\n\
cat /proc/net/ip_tables_names | while read table; do\n\
  $IPTABLES -t $table -L -n | while read c chain rest; do\n\
      if test \"X$c\" = \"XChain\" ; then\n\
        $IPTABLES -t $table -F $chain\n\
      fi\n\
  done\n\
  $IPTABLES -t $table -X\n\
done\n\
\n";
    
    output << endl;;

    return 0;
}

void OSConfigurator_linux24::generateCodeForProtocolHandlers(bool have_nat)
{
    FWOptions* options=fw->getOptionsObject();

    if (options->getBool("load_modules")) {
	output << endl;
	output << "MODULE_DIR=\"/lib/modules/`uname -r`/kernel/net/ipv4/netfilter/\" " << endl;
/*
 * TODO: make better version comparison
 */
        string version=fw->getStr("version");
        if (have_nat)
        {
            output << 
                "MODULES=`(cd $MODULE_DIR; ls *_conntrack_*  *_nat_* | sed 's/\\.o.*$//')`" 
                   << endl;
        } else {
            output << 
                "MODULES=`(cd $MODULE_DIR; ls *_conntrack_*  | sed 's/\\.o.*$//')`" 
                   << endl;
        }

	output << "for module in $(echo $MODULES); do " << endl;
	output << "  if [ -e \"${MODULE_DIR}/${module}.o\" -o -e \"${MODULE_DIR}/${module}.o.gz\" ]; then " << endl;
	output << "    $MODPROBE ${module} ||  exit 1 " << endl;
	output << "  fi " << endl;
	output << "done" << endl;
	output << endl;
	output << endl;
    }
}

void OSConfigurator_linux24::epilog()
{
    FWOptions* options=fw->getOptionsObject();

    try {
	output << "#" << endl;
	output << "#" << endl;

/* Turn on packet forwarding if we have to */

        string s=options->getStr("linux24_ip_forward");
        if (!s.empty())
        {
            if (s=="1" || s=="On" || s=="on") s="1";
            else                              s="0";
            output << "echo " << s << " > /proc/sys/net/ipv4/ip_forward\n\n";
        }
        else
            output << "echo \"$FWD\" > /proc/sys/net/ipv4/ip_forward\n\n";

    } catch (FWException ex) {
	error(ex.toString());
	exit(1);
    }
}
