/* 

                          Firewall Builder

                 Copyright (C) 2002 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_ipf_writers.cc,v 1.22 2002/12/13 08:05:10 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_ipf.hh"


#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/NAT.hh"
#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Host.hh"
#include "fwbuilder/Network.hh"
#include "fwbuilder/Interface.hh"
#include "fwbuilder/IPv4.hh"
#include "fwbuilder/Firewall.hh"


#include <iostream>
#if __GNUC__ > 3 || \
    (__GNUC__ == 3 && (__GNUC_MINOR__ > 2 || (__GNUC_MINOR__ == 2 ) ) )
#  include <streambuf>
#else
#  include <streambuf.h>
#endif
#include <iomanip>
#include <fstream>
#include <sstream>

#include <assert.h>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;



/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */





void NATCompiler_ipf::PrintRule::_printAddr_L(Address  *o, bool print_netmask)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL && ! Interface::cast(o)->isDyn()) 
	mask=Netmask("255.255.255.255");

    if (IPv4::cast(o)!=NULL)
	mask=Netmask("255.255.255.255");

    compiler->output << addr.toString();
    if (print_netmask)
	compiler->output << "/" << mask.getLength();
    compiler->output  << " ";
}

void NATCompiler_ipf::PrintRule::_printAddr_R(Address  *o, bool print_netmask)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL)
	mask=Netmask("255.255.255.255");

    if (IPv4::cast(o)!=NULL)
	mask=Netmask("255.255.255.255");

    if (addr.toString()=="0.0.0.0" && print_netmask && mask.toString()=="255.255.255.255")
        compiler->output  << "0/32 ";
    else
    {
        compiler->output << addr.toString();
        if (print_netmask)
            compiler->output << "/" << mask.getLength();
        compiler->output  << " ";
    }
}


void NATCompiler_ipf::PrintRule::_printAddr_R_LB(RuleElementTDst *tdst)
{
    bool first=true;
    for(list<FWObject*>::iterator i=tdst->begin(); i!=tdst->end(); ++i) 
    {
        FWObject *o= *i;
	FWObject *obj = NULL;
        if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));

        Address *a=Address::cast(obj);

        IPAddress addr=a->getAddress();
        
        if (!first) compiler->output << ",";
        compiler->output << addr.toString();
        first=false;
    }
    compiler->output << " ";
}



void NATCompiler_ipf::PrintRule::_printProtocol(Service *srv)
{
    compiler->output << srv->getProtocolName() << " ";
}

void NATCompiler_ipf::PrintRule::_printPort(Service *srv,bool eq)
{
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int drs=srv->getInt("dst_range_start");
	if (drs!=0)   
        {
            compiler->output << "port ";
            if (eq)
                compiler->output << "= ";
            compiler->output<< drs << " ";
        }
    }
}


NATCompiler_ipf::PrintRule::PrintRule(const std::string &name) : NATCompiler_pf::PrintRule(name)
{ 
}

bool NATCompiler_ipf::PrintRule::processNext()
{
    NATRule *rule=getNext(); if (rule==NULL) return false;

    tmp_queue.push_back(rule);

    string rl=rule->getLabel();
    if (rl!=current_rule_label) {

        compiler->output << "# " << endl;
        compiler->output << "# Rule  " << rl << endl;

        string    comm=rule->getComment();
        string::size_type c1,c2;
        c1=0;
        while ( (c2=comm.find('\n',c1))!=string::npos ) {
            compiler->output << "# " << comm.substr(c1,c2-c1) << endl;
            c1=c2+1;
        }
        compiler->output << "# " << comm.substr(c1) << endl;
        compiler->output << "# " << endl;

        current_rule_label=rl;
    }


    Address  *osrc=rule->getOSrc()->getFirst(true);  assert(osrc);
    Address  *odst=rule->getODst()->getFirst(true);  assert(odst);
    Service  *osrv=rule->getOSrv()->getFirst(true);  assert(osrv);

    Address  *tsrc=rule->getTSrc()->getFirst(true);  assert(tsrc);
    Address  *tdst=rule->getTDst()->getFirst(true);  assert(tdst);
    Service  *tsrv=rule->getTSrv()->getFirst(true);  assert(tsrv);

    Interface *iface=
	Interface::cast( rule->getRoot()->getById(rule->getInterfaceId() ,true) );

    if (rule->getRuleType()==NATRule::NONAT) 
    {
	compiler->output  << "map "
                          << iface->getName() 
                          << " ";
	compiler->output  << "from ";
	_printAddr_L( osrc );
	compiler->output  << "to ";
	_printAddr_L( odst );

	_printPort(osrv,true);

	compiler->output  << "-> 0/0";
//	_printAddr_R( tsrc );
	compiler->output  << " ";

	if (rule->getBool("ftp_proxy"))    compiler->output << "proxy port " << FTP_PORT    << " ftp/tcp ";
	if (rule->getBool("rcmd_proxy"))   compiler->output << "proxy port " << RCMD_PORT   << " rcmd/tcp ";
	if (rule->getBool("raudio_proxy")) compiler->output << "proxy port " << RAUDIO_PORT << " raudio/tcp ";
	if (rule->getBool("h323_proxy"))   compiler->output << "proxy port " << H323_PORT   << " h323/tcp ";
	if (rule->getBool("ipsec_proxy"))  compiler->output << "proxy port " << ISAKMP_PORT << " ipsec/udp ";

	compiler->output  << endl;
    }

    if (rule->getRuleType()==NATRule::SNAT) 
    {
	compiler->output  << "map "
                          << iface->getName() 
                          << " ";
	compiler->output  << "from ";
	_printAddr_L( osrc );
	compiler->output  << "to ";
	_printAddr_L( odst );

	_printPort(osrv,true);

	compiler->output  << "-> ";
	_printAddr_R( tsrc );
	compiler->output  << " ";

	if (rule->getBool("ftp_proxy"))    compiler->output << "proxy port " << FTP_PORT    << " ftp/tcp ";
	if (rule->getBool("rcmd_proxy"))   compiler->output << "proxy port " << RCMD_PORT   << " rcmd/tcp ";
	if (rule->getBool("raudio_proxy")) compiler->output << "proxy port " << RAUDIO_PORT << " raudio/tcp ";
	if (rule->getBool("h323_proxy"))   compiler->output << "proxy port " << H323_PORT   << " h323/tcp ";
	if (rule->getBool("ipsec_proxy"))  compiler->output << "proxy port " << ISAKMP_PORT << " ipsec/udp ";

	compiler->output  << endl;
    }

    if (rule->getRuleType()==NATRule::DNAT) 
    {
	compiler->output  << "rdr " 
                          << iface->getName() 
                          << " ";

        compiler->output  << "from ";
        _printAddr_L( osrc , true );
        compiler->output  << "to ";
        _printAddr_L( odst , true );
        
	_printPort(osrv,true);
	compiler->output  << "-> ";
	_printAddr_R( tdst , false );
	_printPort(tsrv,false);
        if ( ! osrv->isAny()) _printProtocol(osrv);
	compiler->output  << endl;
    }

    if (rule->getRuleType()==NATRule::LB) 
    {
	compiler->output  << "rdr " 
                          << iface->getName() 
                          << " ";
        compiler->output  << "from ";
        _printAddr_L( osrc , true );
        compiler->output  << "to ";
        _printAddr_L( odst , true );
	_printPort(osrv,true);
	compiler->output  << "-> ";
	_printAddr_R_LB( rule->getTDst() );
	_printPort(tsrv,false);
        if ( ! osrv->isAny()) _printProtocol(osrv);
        compiler->output << " round-robin ";
	compiler->output << endl;
    }

    if (rule->getRuleType()==NATRule::Redirect) 
    {
	compiler->output  << "rdr " 
                          << iface->getName() 
                          << " ";
	_printAddr_L( odst , true );
	_printPort(osrv,false);
	compiler->output  << "-> ";
	_printAddr_R( tdst , false );
	_printPort(tsrv,false);
	_printProtocol(osrv);
	compiler->output  << endl;
    }

    return true;
}

