/* 

                          Firewall Builder

                 Copyright (C) 2002 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_pf.cc,v 1.49 2002/11/30 21:04:40 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "PolicyCompiler_pf.hh"

#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"
#include "fwbuilder/Interface.hh"
#include "fwbuilder/Firewall.hh"

#include <iostream>
#include <iomanip>

#include <assert.h>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string PolicyCompiler_pf::myPlatformName() { return "pf"; }

int PolicyCompiler_pf::prolog()
{


    if (fw->getStr("platform")!="pf") 
	abort(_("Unsupported platform ") + fw->getStr("platform") );

    return PolicyCompiler::prolog();
}

bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    PolicyRule     *r;
    RuleElementSrc *src=rule->getSrc();    assert(src);

    if (src->size()==1) {
	tmp_queue.push_back(rule);
	return true;
    }
    FWObject       *fw_in_src=NULL;
    vector<FWObject*> cl;
    for (FWObject::iterator i1=src->begin(); i1!=src->end(); ++i1) {

	FWObject *o   = *i1;
	FWObject *obj = NULL;
//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
	if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
	if (obj==NULL)  throw FWException(_("Broken Src object in rule: ")+rule->getLabel());

	if (obj->getId()==compiler->getFwId()) {
	    fw_in_src=o;   // can not remove right now because remove invalidates iterator

	    RuleElementSrc *nsrc;

	    r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
	    compiler->temp_ruleset->add(r);
	    r->duplicate(rule);
	    nsrc=r->getSrc();
	    nsrc->clearChildren();
	    nsrc->setAnyElement();
	    nsrc->addRef( compiler->fw );
	    tmp_queue.push_back(r);
	}
    }
    if (fw_in_src!=NULL) src->remove( fw_in_src );

    tmp_queue.push_back(rule);
    return true;
}


bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    PolicyRule     *r;
    RuleElementDst *dst=rule->getDst();    assert(dst);

    if (dst->size()==1) {
	tmp_queue.push_back(rule);
	return true;
    }

    FWObject       *fw_in_dst=NULL;
    vector<FWObject*> cl;
    for (FWObject::iterator i1=dst->begin(); i1!=dst->end(); ++i1) {

	FWObject *o   = *i1;
	FWObject *obj = NULL;
//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
	if (FWReference::cast(o)!=NULL) obj=compiler->getCachedObj(o->getStr("ref"));
	if (obj==NULL) throw FWException(_("Broken Dst in rule: ")+rule->getLabel());

	if (obj->getId()==compiler->getFwId()) {
	    fw_in_dst=o;   // can not remove right now because remove invalidates iterator

	    RuleElementDst *ndst;

	    r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
	    compiler->temp_ruleset->add(r);
	    r->duplicate(rule);
	    ndst=r->getDst();
	    ndst->clearChildren();
	    ndst->setAnyElement();
	    ndst->addRef( compiler->fw );
	    tmp_queue.push_back(r);
	}
    }
    if (fw_in_dst!=NULL) dst->remove( fw_in_dst );

    tmp_queue.push_back(rule);
    return true;
}



bool PolicyCompiler_pf::fillDirection::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;
    tmp_queue.push_back(rule);

    if (rule->getDirectionAsString()=="") {

	if ( compiler->getCachedFwOpt()->getBool("pass_all_out") )  rule->setDirection( PolicyRule::Inbound );
	else {
	    rule->setDirection( PolicyRule::Both );

	    Address  *src=rule->getSrc()->getFirst(true);
	    Address  *dst=rule->getDst()->getFirst(true);
	    string    fwid=compiler->getFwId();

            if (!src->isAny() && !dst->isAny() &&
                compiler->complexMatch(compiler->fw, src) &&
                compiler->complexMatch(compiler->fw, dst)) return true;

            if (!src->isAny() && compiler->complexMatch(compiler->fw, src)) rule->setDirection( PolicyRule::Outbound );
            if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst)) rule->setDirection( PolicyRule::Inbound );
	}
    }
    return true;
}


void PolicyCompiler_pf::addDefaultPolicyRule()
{
    if ( getCachedFwOpt()->getBool("pass_all_out") ) {
	PolicyRule *r;

	r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Accept);
	r->setLogging(false);
	r->setDirection(PolicyRule::Outbound);
	r->setPosition(10000);
        r->setComment("   fallback rule ");
        r->setHidden(true);
        r->setFallback(true);
        r->setLabel("fallback rule");
	combined_ruleset->push_back(r);

	r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Deny);
	r->setLogging(false);
	r->setDirection(PolicyRule::Inbound);
	r->setPosition(10001);
        r->setComment("   fallback rule ");
        r->setHidden(true);
        r->setFallback(true);
        r->setLabel("fallback rule");
	combined_ruleset->push_back(r);
    } else {
	PolicyRule *r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Deny);
	r->setLogging(false);
	r->setDirection(PolicyRule::Both);
	r->setPosition(10000);
        r->setComment("   fallback rule ");
        r->setHidden(true);
        r->setFallback(true);
        r->setLabel("fallback rule");
	combined_ruleset->push_back(r);
    }
}

bool PolicyCompiler_pf::SpecialServices::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;
    tmp_queue.push_back(rule);

    RuleElementSrv *srv=rule->getSrv();

    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
	Service *s=Service::cast( o );
	assert(s);

	if (IPService::cast(s)!=NULL  && rule->getAction()==PolicyRule::Accept) {
	    if (s->getBool("rr")        ||
		s->getBool("ssrr")      ||
		s->getBool("ts") )
		rule->setBool("allow_opts",true);
	}
    }
    return true;
}

bool PolicyCompiler_pf::SplitDirection::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    if (rule->getDirection()==PolicyRule::Both) {
	PolicyRule *r= PolicyRule::cast(
	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setDirection(PolicyRule::Inbound);
	tmp_queue.push_back(r);

	r= PolicyRule::cast(
	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setDirection(PolicyRule::Outbound);
	tmp_queue.push_back(r);

    } else 
	tmp_queue.push_back(rule);

    return true;
}

bool PolicyCompiler_pf::ProcessScrubOption::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    FWOptions *ruleopt =rule->getOptionsObject();


    if ( ruleopt->getBool("scrub") ) {

	if (rule->getAction()!=PolicyRule::Accept) {
	    ruleopt->setBool("scrub",false);
	    tmp_queue.push_back(rule);

	    throw FWException(_("Rule option 'scrub' is supported only for rules with action 'Accept'. Rule: ")+rule->getLabel());

	    return true;
	}

	PolicyRule *r= PolicyRule::cast(
	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Scrub);
	r->getOptionsObject()->setBool("scrub",false);
	tmp_queue.push_back(r);

	ruleopt->setBool("scrub",false);
	tmp_queue.push_back(rule);

	return true;
    }

/* if service is ip_fragment and action is 'Deny', then add rule with scrub */

    Service *srv=rule->getSrv()->getFirst(true);    assert(srv);

    if ( (srv->getBool("short_fragm") || srv->getBool("fragm")) &&
	 ( rule->getAction()==PolicyRule::Deny || rule->getAction()==PolicyRule::Reject) ) {

	PolicyRule *r= PolicyRule::cast(
	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Scrub);
	r->getOptionsObject()->setBool("scrub",false);
	tmp_queue.push_back(r);

	return true;
    }

    tmp_queue.push_back(rule);
    return true;
}

bool PolicyCompiler_pf::MACFiltering::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;
    tmp_queue.push_back(rule);

    bool src_usemac=false;
    bool dst_usemac=false;

    RuleElementSrc *src=rule->getSrc();
    RuleElementDst *dst=rule->getDst();
    FWOptions *hostopt;

    for (FWObject::iterator i=src->begin(); i!=src->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
        if (Host::cast(o)!=NULL) {
            hostopt=Host::cast(o)->getOptionsObject();
            src_usemac=hostopt->getBool("use_mac_addr_filter");
        }
    }

    for (FWObject::iterator i=dst->begin(); i!=dst->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
        if (Host::cast(o)!=NULL) {
            hostopt=Host::cast(o)->getOptionsObject();
            dst_usemac=hostopt->getBool("use_mac_addr_filter");
        }
    }

    if ( src_usemac || dst_usemac )
	throw FWException(_("MAC address matching is not spupported. Rule: ")+rule->getLabel());

    return true;
}


bool PolicyCompiler_pf::setQuickFlag::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;
    tmp_queue.push_back(rule);

    if ( rule->getAction()!=PolicyRule::Scrub ) rule->setBool("quick",true);

    return true;
}

bool PolicyCompiler_pf::doSrcNegation::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    RuleElementSrc *src=rule->getSrc();

    if (src->getNeg()) {
        RuleElementSrc *nsrc;
	PolicyRule     *r;

	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
	else	                                    r->setAction(PolicyRule::Accept);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	r->setBool("quick",true);
        r->setLogging(false);
	tmp_queue.push_back(r);

	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	nsrc->clearChildren();
	nsrc->setAnyElement();
	r->setBool("quick",true);
	tmp_queue.push_back(r);

	return true;
    }
    tmp_queue.push_back(rule);
    return true;
}

bool PolicyCompiler_pf::doDstNegation::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    RuleElementDst *dst=rule->getDst();

    if (dst->getNeg()) {
        RuleElementDst *ndst;
	PolicyRule     *r;

	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
	else	                                    r->setAction(PolicyRule::Accept);
        ndst=r->getDst();
        ndst->setNeg(false);
	r->setBool("quick",true);
        r->setLogging(false);
	tmp_queue.push_back(r);

	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
	compiler->temp_ruleset->add(r);
	r->duplicate(rule);
        ndst=r->getDst();
        ndst->setNeg(false);
	ndst->clearChildren();
	ndst->setAnyElement();
	r->setBool("quick",true);
	tmp_queue.push_back(r);

	return true;
    }
    tmp_queue.push_back(rule);
    return true;
}

bool PolicyCompiler_pf::doSrvNegation::processNext()
{
    PolicyRule *rule=getNext(); if (rule==NULL) return false;

    RuleElementSrv *srv=rule->getSrv();

    if (srv->getNeg()) {
	throw FWException(_("Negation in Srv is not implemented. Rule: ")+rule->getLabel());
	return true;
    }
    tmp_queue.push_back(rule);
    return true;
}




void PolicyCompiler_pf::compile()
{
    cout << _(" Compiling policy for ") << fw->getName() << " ..." <<  endl << flush;

    try {

	Compiler::compile();

	addDefaultPolicyRule();

        if ( fw->getOptionsObject()->getBool ("check_shading") ) 
        {
            add( new Begin                       (" Detecting rule shading"                ) );
            add( new printTotalNumberOfRules     (                                         ) );
            add( new ExpandGroups                ("expand groups"                          ) );
            add( new eliminateDuplicatesInSRC    ("eliminate duplicates in SRC"            ) );
            add( new eliminateDuplicatesInDST    ("eliminate duplicates in DST"            ) );
            add( new eliminateDuplicatesInSRV    ("eliminate duplicates in SRV"            ) );
            add( new ExpandMultipleAddressesInSRC("expand objects with multiple addresses in SRC" ) );
            add( new ExpandMultipleAddressesInDST("expand objects with multiple addresses in DST" ) );
            add( new ConvertToAtomic             ("convert to atomic rules"                ) );
            add( new DetectShading               ("Detect shading"                         ) );
            add( new simplePrintProgress         (                                         ) );

            runRuleProcessors();
            deleteRuleProcessors();
        }

        add( new Begin(                 "Start processing rules"                   ) );
        add( new printTotalNumberOfRules() );
        add( new MACFiltering(          "verify for MAC address filtering"         ) );

        add( new emptyGroupsInSrc(           "check for empty groups in SRC"         ) );
        add( new emptyGroupsInDst(           "check for empty groups in DST"         ) );
        add( new emptyGroupsInSrv(           "check for empty groups in SRV"         ) );

        add( new doSrcNegation(         "process negation in Src"                  ) );
        add( new doDstNegation(         "process negation in Dst"                  ) );
//	add( new doSrvNegation(         "process negation in Srv"                  ) );
        add( new ExpandGroups(          "expand groups"                            ) );
        add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"            ) );
        add( new eliminateDuplicatesInDST("eliminate duplicates in DST"            ) );
        add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"            ) );

	add( new splitIfFirewallInSrc(  "split rule if firewall is in Src"         ) );
	add( new splitIfFirewallInDst(  "split rule if firewall is in Dst"         ) );
	add( new fillDirection(         "determine directions"                     ) );
	add( new SplitDirection(        "split rules with direction 'both'"        ) );
	add( new ExpandMultipleAddresses( "expand objects with multiple addresses" ) );
	add( new addressRanges(         "expand address range objects"             ) );
	add( new splitServices(         "split rules with different protocols"     ) );
	add( new separateTCPWithFlags(  "separate TCP services with flags"         ) );
	add( new verifyCustomServices(  "verify custom services for this platform" ) );
	add( new ProcessScrubOption(    "process 'scrub' option"                   ) );
	add( new SpecialServices(       "check for special services"               ) );
	add( new setQuickFlag(          "set 'quick' flag"                         ) );
        add( new checkForZeroAddr(      "check for zero addresses"                 ) );
        add( new PrintRule(             "generate pf code"                         ) );
//        if (check_shading)
//            add( new DetectShading(     "Detect shading"                           ) );
        add( new simplePrintProgress() );

        runRuleProcessors();

    } catch (FWException &ex) {
	error(ex.toString());
	exit(1);
    }
}


void PolicyCompiler_pf::epilog()
{
    cout << _(" Policy compiled successfully \n") << flush;
}
