/* 

                          Firewall Builder

                 Copyright (C) 2000 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PIXAdvancedFWDialog.cc,v 1.1 2002/11/26 05:24:37 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"
#include "PIXAdvancedFWDialog.hh"
#include "fwbuilder/Resources.hh"

#include <gtk--.h>
#include <sstream>

using namespace libfwbuilder;

PIXAdvancedFWDialog::PIXAdvancedFWDialog(Firewall *_fo)
{ 
    fwobj=_fo;

    ((Gtk::Label*)(fragguard->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(connection_timewait->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(route_dnat->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(route_dnat->get_child()))->set_justify(GTK_JUSTIFY_LEFT);

    ((Gtk::Label*)(nodnsalias_inbound->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(nodnsalias_outbound->get_child()))->set_alignment(0.0,1.0);

    ok_button->grab_focus();

    log_facilities[""    ]="      "  ;
    log_facilities["16"  ]="LOCAL0"  ;
    log_facilities["17"  ]="LOCAL1"  ;
    log_facilities["18"  ]="LOCAL2"  ;
    log_facilities["19"  ]="LOCAL3"  ;
    log_facilities["20"  ]="LOCAL4"  ;
    log_facilities["21"  ]="LOCAL5"  ;
    log_facilities["22"  ]="LOCAL6"  ;
    log_facilities["23"  ]="LOCAL7"  ;

    log_levels["" ]="        "  ;
    log_levels["0"]="0 - System Unusable                  ";
    log_levels["1"]="1 - Take Immediate Action            ";
    log_levels["2"]="2 - Critical Condition               ";
    log_levels["3"]="3 - Error Message                    ";
    log_levels["4"]="4 - Warning Message                  ";
    log_levels["5"]="5 - Normal but significant condition ";
    log_levels["6"]="6 - Informational                    ";
    log_levels["7"]="7 - Debug Message                    ";


		
		
    syslog_facility->set_menu(log_facilities);
    syslog_facility->show_all();
    
    logging_trap_level->set_menu(log_levels);
    logging_trap_level->show_all();

    logging_buffered_level->set_menu(log_levels);
    logging_buffered_level->show_all();

    logging_console_level->set_menu(log_levels);
    logging_console_level->show_all();


    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    string vers="version_"+fwobj->getStr("version");
    
    string lst=Resources::platform_res["pix"]->getResourceStr(
            "/FWBuilderResources/Target/options/"+vers+"/fixups/list");

    string::size_type i,j, k;
    i=0;
    while ( i<lst.size() )
    {
        j=lst.find(",",i);

        string s=lst.substr(i,j-i);
        string l=s;
        if ( (k=l.find("_"))!=string::npos ) l.replace(k,1,1,' ');
        
        fixup_names.push_back( l );
        fixup_xml_elements[l]=s;

        if (j==string::npos) break;
        i=j+1;
    }

    new_fixup_type->set_popdown_strings( fixup_names );
}

void  PIXAdvancedFWDialog::wrk2dlg()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    syslog_host->set_text(        options->getStr("pix_syslog_host"        ) );
    syslog_queue_size->set_value( options->getInt( "pix_syslog_queue_size" ) );
    syslog_facility->set_active_item_by_value( options->getStr("pix_syslog_facility") );
    logging_trap_level->set_active_item_by_value( options->getStr("pix_logging_trap_level") );

    logging_buffered->set_active( options->getBool("pix_logging_buffered") );
    logging_buffered_level->set_active_item_by_value( options->getStr("pix_logging_buffered_level") );

    logging_console->set_active( options->getBool("pix_logging_console") );
    logging_console_level->set_active_item_by_value( options->getStr("pix_logging_console_level") );

    logging_timestamp->set_active( options->getBool("pix_logging_timestamp") );


    
    xlate_hh->set_value(          options->getInt("xlate_hh"    ) );
    xlate_mm->set_value(          options->getInt("xlate_mm"    ) );
    xlate_ss->set_value(          options->getInt("xlate_ss"    ) );

    conn_hh->set_value(           options->getInt("conn_hh"     ) );
    conn_mm->set_value(           options->getInt("conn_mm"     ) );
    conn_ss->set_value(           options->getInt("conn_ss"     ) );

    udp_hh->set_value(            options->getInt("udp_hh"        ) );
    udp_mm->set_value(            options->getInt("udp_mm"        ) );
    udp_ss->set_value(            options->getInt("udp_ss"        ) );

    rpc_hh->set_value(            options->getInt("rpc_hh"        ) );
    rpc_mm->set_value(            options->getInt("rpc_mm"        ) );
    rpc_ss->set_value(            options->getInt("rpc_ss"        ) );

    h323_hh->set_value(           options->getInt("h323_hh"      ) );
    h323_mm->set_value(           options->getInt("h323_mm"      ) );
    h323_ss->set_value(           options->getInt("h323_ss"      ) );

    sip_hh->set_value(            options->getInt("sip_hh"        ) );
    sip_mm->set_value(            options->getInt("sip_mm"        ) );
    sip_ss->set_value(            options->getInt("sip_ss"        ) );

    sip_media_hh->set_value(      options->getInt("sip_media_hh"  ) );
    sip_media_mm->set_value(      options->getInt("sip_media_mm"  ) );
    sip_media_ss->set_value(      options->getInt("sip_media_ss"  ) );
                                                                  
    half_closed_hh->set_value(    options->getInt("half_closed_hh") );
    half_closed_mm->set_value(    options->getInt("half_closed_mm") );
    half_closed_ss->set_value(    options->getInt("half_closed_ss") );

    uauth_hh->set_value(         options->getInt("uauth_hh"    ) );
    uauth_mm->set_value(         options->getInt("uauth_mm"    ) );
    uauth_ss->set_value(         options->getInt("uauth_ss"    ) );
    uauth_abs->set_active(       options->getBool("uauth_abs"  ) );
    uauth_inact->set_active(     options->getBool("uauth_inact") );

    telnet_timeout->set_value( options->getInt("pix_telnet_timeout") );
    ssh_timeout->set_value( options->getInt("pix_ssh_timeout") );

    resetinbound->set_active( options->getBool("pix_resetinbound"));
    resetoutside->set_active( options->getBool("pix_resetoutside"));

    connection_timewait->set_active( options->getBool("pix_connection_timewait"));
    fragguard->set_active(           options->getBool("pix_fragguard"));
    floodguard->set_active(          options->getBool("pix_floodguard"));
    route_dnat->set_active(          options->getBool("pix_route_dnat"));
    nodnsalias_inbound->set_active(  options->getBool("pix_nodnsalias_inbound"));
    nodnsalias_outbound->set_active( options->getBool("pix_nodnsalias_outbound"));

    loadFixups();
}

void  PIXAdvancedFWDialog::dlg2wrk()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    options->setStr("pix_syslog_host",       syslog_host->get_text() );
    options->setInt("pix_syslog_queue_size", syslog_queue_size->get_value_as_int() );

    options->setStr("pix_syslog_facility"  ,  syslog_facility->get_value()    );
    options->setStr("pix_logging_trap_level", logging_trap_level->get_value() );

    options->setBool("pix_logging_buffered",       logging_buffered->get_active()      );
    options->setStr( "pix_logging_buffered_level", logging_buffered_level->get_value() );

    options->setBool("pix_logging_console",       logging_console->get_active()      );
    options->setStr( "pix_logging_console_level", logging_console_level->get_value() );

    options->setBool("pix_logging_timestamp",     logging_timestamp->get_active()    );



    options->setInt("xlate_hh"         , xlate_hh->get_value_as_int() );
    options->setInt("xlate_mm"         , xlate_mm->get_value_as_int() );
    options->setInt("xlate_ss"         , xlate_ss->get_value_as_int() );
                                             
    options->setInt("conn_hh"          , conn_hh->get_value_as_int() );
    options->setInt("conn_mm"          , conn_mm->get_value_as_int() );
    options->setInt("conn_ss"          , conn_ss->get_value_as_int() );
                                                                     
    options->setInt("udp_hh"           , udp_hh->get_value_as_int() );
    options->setInt("udp_mm"           , udp_mm->get_value_as_int() );
    options->setInt("udp_ss"           , udp_ss->get_value_as_int() );
                                                                    
    options->setInt("rpc_hh"           , rpc_hh->get_value_as_int() );
    options->setInt("rpc_mm"           , rpc_mm->get_value_as_int() );
    options->setInt("rpc_ss"           , rpc_ss->get_value_as_int() );
                                                                     
    options->setInt("h323_hh"          , h323_hh->get_value_as_int() );
    options->setInt("h323_mm"          , h323_mm->get_value_as_int() );
    options->setInt("h323_ss"          , h323_ss->get_value_as_int() );
                                                                     
    options->setInt("sip_hh"           , sip_hh->get_value_as_int() );
    options->setInt("sip_mm"           , sip_mm->get_value_as_int() );
    options->setInt("sip_ss"           , sip_ss->get_value_as_int() );
                                            
    options->setInt("sip_media_hh"     , sip_media_hh->get_value_as_int() );
    options->setInt("sip_media_mm"     , sip_media_mm->get_value_as_int() );
    options->setInt("sip_media_ss"     , sip_media_ss->get_value_as_int() );

    options->setInt("half_closed_hh"   , half_closed_hh->get_value_as_int() );
    options->setInt("half_closed_mm"   , half_closed_mm->get_value_as_int() );
    options->setInt("half_closed_ss"   , half_closed_ss->get_value_as_int() );

    options->setInt("uauth_hh"        , uauth_hh->get_value_as_int() );
    options->setInt("uauth_mm"        , uauth_mm->get_value_as_int() );
    options->setInt("uauth_ss"        , uauth_ss->get_value_as_int() );
    options->setBool("uauth_abs"      , uauth_abs->get_active() );
    options->setBool("uauth_inact"    , uauth_inact->get_active() );

    options->setInt("pix_telnet_timeout"   , telnet_timeout->get_value_as_int() );
    options->setInt("pix_ssh_timeout"      , ssh_timeout->get_value_as_int() );

    options->setBool("pix_resetinbound"        ,resetinbound->get_active());
    options->setBool("pix_resetoutside"        ,resetoutside->get_active());

    options->setBool("pix_connection_timewait" ,connection_timewait->get_active());
    options->setBool("pix_fragguard"           ,fragguard->get_active());
    options->setBool("pix_floodguard"          ,floodguard->get_active());
    options->setBool("pix_route_dnat"          ,route_dnat->get_active());
    options->setBool("pix_nodnsalias_inbound"  ,nodnsalias_inbound->get_active());
    options->setBool("pix_nodnsalias_outbound" ,nodnsalias_outbound->get_active());

    saveFixups();
}

bool PIXAdvancedFWDialog::run()
{
    ok=false;

// Load data
    wrk2dlg();

    show();
    Gtk::Main::grab_add(*this);
    Gtk::Main::run();
    Gtk::Main::grab_remove(*this);
    hide();

    if (ok) {
// save data
        dlg2wrk();
    }
    return ok;
}

void PIXAdvancedFWDialog::on_ok_clicked()
{   
    ok=true;
    Gtk::Main::quit();
}

void PIXAdvancedFWDialog::on_cancel_clicked()
{   
    ok=false;
    Gtk::Main::quit();
}

void PIXAdvancedFWDialog::setDefaultTimeoutValue(const string &option)
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    string vers="version_"+fwobj->getStr("version");

    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/"+option+"_hh");
    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/"+option+"_mm");
    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/"+option+"_ss");
}

void PIXAdvancedFWDialog::on_default_timeouts_clicked()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);
    
    string vers="version_"+fwobj->getStr("version");

    setDefaultTimeoutValue("xlate"       );
    setDefaultTimeoutValue("conn"        );
    setDefaultTimeoutValue("udp"         );
    setDefaultTimeoutValue("rpc"         );
    setDefaultTimeoutValue("h323"        );
    setDefaultTimeoutValue("sip"         );
    setDefaultTimeoutValue("sip_media"   );
    setDefaultTimeoutValue("half_closed" );
    setDefaultTimeoutValue("uauth"       );

    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/uauth_abs");
    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/uauth_inact");

    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/telnet_timeout");
    Resources::platform_res["pix"]->setDefaultOption(options,
                "/FWBuilderResources/Target/options/"+vers+"/timeouts/ssh_timeout");
    
    wrk2dlg();
}

FixupWidget* PIXAdvancedFWDialog::createDefaultFixupWidget(const string &name)
{
    string vers="version_"+fwobj->getStr("version");

    string r="/FWBuilderResources/Target/options/"+vers+"/fixups/"+fixup_xml_elements[name];

    int p1=Resources::platform_res["pix"]->getResourceInt( r+"/port1");
    int p2=Resources::platform_res["pix"]->getResourceInt( r+"/port2");
    string on=Resources::platform_res["pix"]->getResourceStr( r+"/option/name");
    string ov=Resources::platform_res["pix"]->getResourceStr( r+"/option/val");

    return new FixupWidget( name , false , p1 ,p2 ,on, ov=="0");
}

void PIXAdvancedFWDialog::on_default_fixups_clicked()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    clearFixupWidgets();    

    class FixupWidget *fw;

    for (list<string>::iterator i=fixup_names.begin(); i!=fixup_names.end(); ++i)
        addFixupWidget( createDefaultFixupWidget( *i ) );
}

void PIXAdvancedFWDialog::clearFixupWidgets()
{
    Gtk::Widget *w;
    Gtk::Box_Helpers::BoxList::iterator i;

    while ( (i=fixup_vbox->children().begin())!=fixup_vbox->children().end() )
        fixup_vbox->remove( *((*i)->get_widget()) );
}

void PIXAdvancedFWDialog::addFixupWidget(FixupWidget *fw)
{
    manage(fw);
    fixup_vbox->pack_start( *fw,false,false );
}

void PIXAdvancedFWDialog::addFixupWidget(const std::string &n,bool dis,
                        int  p1,int p2,const std::string &on,bool ov )
{
    addFixupWidget( new FixupWidget( n , dis , p1 ,p2 ,on, ov) );
}

void PIXAdvancedFWDialog::addFixupWidget(const std::string &name,
                                         const std::string &s)
{
    bool   no;
    int    p1,p2;
    string on;
    bool   ov;
    istringstream str(s);

    str >> no >> p1 >> p2 >> on >> ov;

    if (on=="nil")    on="";

    addFixupWidget( name , no , p1 ,p2 ,on, ov);
}

void PIXAdvancedFWDialog::loadFixups()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);
    
    clearFixupWidgets();    

    class FixupWidget *fw;

    for (list<string>::iterator n=fixup_names.begin(); n!=fixup_names.end(); ++n)
    {
        string f=options->getStr(fixup_xml_elements[(*n)]+"_fixup");
        if (!f.empty())
        {
            string::size_type i,j;
            i=0;
            while ( (j=f.find(",",i))!=string::npos )
            {
                addFixupWidget((*n),f.substr(i,j-i));
                i=j+1;
            }
            addFixupWidget((*n),f.substr(i));
        }
    }
}

void PIXAdvancedFWDialog::saveFixups()
{
    FWOptions *options=fwobj->getOptionsObject();
    assert(options!=NULL);

    for (list<string>::iterator i=fixup_names.begin(); i!=fixup_names.end(); ++i)
    {
        string fxe=fixup_xml_elements[(*i)];
        if (! options->getStr(fxe+"_fixup").empty())
            options->remStr(fxe+"_fixup");
    }

    map<string,string> fixup_strings;

    for (Gtk::Box_Helpers::BoxList::iterator i=fixup_vbox->children().begin(); i!=fixup_vbox->children().end(); ++i)
    {
        FixupWidget *fw=dynamic_cast<FixupWidget*>((*i)->get_widget());

        string name=fw->getName();
        bool   dis =fw->isDisabled();
        int    p1  =fw->getPort1();
        int    p2  =fw->getPort2();
        string on  =fw->getOptionName();
        bool   ov  =fw->getOption();

        string fxe=fixup_xml_elements[name];

        fxe=fxe+"_fixup";
        if (on.empty()) on="nil";

        ostringstream str;
        str << dis << " " << p1 << " " << p2 << " " << on << " " << ov;

        if (!fixup_strings[fxe].empty()) 
            fixup_strings[fxe]=fixup_strings[fxe]+","+str.str();
        else
            fixup_strings[fxe]=str.str();
    }

    for (map<string,string>::iterator i=fixup_strings.begin(); i!=fixup_strings.end(); ++i)
        options->setStr( i->first , i->second );
}


void PIXAdvancedFWDialog::on_add_fixup_clicked()
{
    string fixup_type=new_fixup_type->get_entry()->get_text();

    if (!fixup_type.empty())
    {
        FixupWidget *nfw= createDefaultFixupWidget( fixup_type );

        int pos=0;
        for (Gtk::Box_Helpers::BoxList::iterator n=fixup_vbox->children().begin(); 
             n!=fixup_vbox->children().end(); ++n,++pos)
        {
            FixupWidget *fw=dynamic_cast<FixupWidget*>((*n)->get_widget());
            if (fw->getName()>=fixup_type)
            {
                while (n!=fixup_vbox->children().end())
                {
                    fw=dynamic_cast<FixupWidget*>((*n)->get_widget());
                    if (fw->getName()!=fixup_type) break;
                    ++n; ++pos;
                }
                addFixupWidget(nfw);
                fixup_vbox->reorder_child(*nfw,pos);
                return;
            }
        }
        addFixupWidget(nfw);
    }
}
